/**
 *    Free Tide Mini is a free harmonic tide predictor
 *    Copyright (C) 2008  Regis COSNIER
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package freetide.mini;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

/**
 *
 * @author Dgis
 */
public class Configuration
{
	public static final byte LANGUAGE_ENGLISH = 0;
	public static final byte LANGUAGE_FRENCH = 1;

	public static final byte CLOCK_12H = 0;
	public static final byte CLOCK_24H = 1;

	public byte language = LANGUAGE_ENGLISH;
	public String harmonicFilePath = "/Harmonic.idx";
	public int lastStation = 1;
	public int unit = 0;
	public int clockType = CLOCK_24H;

	private RecordStore recordStore = null;
	
	public Configuration()
	{
		try
		{
		    recordStore = RecordStore.openRecordStore("FreeTideMini", true);
		}
		catch (RecordStoreException rse)
		{
			System.out.println(rse);
			rse.printStackTrace();
		}
	}

	public Configuration(Configuration c)
	{
		set(c);
	}
	
	public void set(Configuration c)
	{
		this.language = c.language;
		this.harmonicFilePath = c.harmonicFilePath;
		this.lastStation = c.lastStation;
		this.unit = c.unit;
		this.clockType = c.clockType;
	}
	
	int recordId = -1;
	
	public void load()
	{
		try
		{
			RecordEnumeration re = recordStore.enumerateRecords(null, null, true);
			if(re.hasNextElement())
			{
				recordId = re.nextRecordId();
				ByteArrayInputStream bais = new ByteArrayInputStream(recordStore.getRecord(recordId));
				DataInputStream inputStream = new DataInputStream(bais);
				
				language = inputStream.readByte();
				harmonicFilePath = inputStream.readUTF();
				lastStation = inputStream.readInt();
				unit = inputStream.readByte();
				clockType = inputStream.readByte();
			}
		}
		catch (EOFException eofe)
		{
			System.out.println(eofe);
			eofe.printStackTrace();
		}
		catch (RecordStoreException rse)
		{
			System.out.println(rse);
			rse.printStackTrace();
		}
		catch (IOException ioe)
		{
			System.out.println(ioe);
			ioe.printStackTrace();
		}
	}
	
	public void save()
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream outputStream = new DataOutputStream(baos);
		try
		{
			outputStream.writeByte(language);
			outputStream.writeUTF(harmonicFilePath);
			outputStream.writeInt(lastStation);
			outputStream.writeByte(unit);
			outputStream.writeByte(clockType);
		}
		catch (IOException ioe)
		{
			System.out.println(ioe);
			ioe.printStackTrace();
		}

		// Extract the byte array
		byte[] b = baos.toByteArray();
		// Add it to the record store
		try
		{
			if(recordId > -1)
				recordStore.setRecord(recordId, b, 0, b.length);
			else
				recordId = recordStore.addRecord(b, 0, b.length);
		}
		catch (RecordStoreException rse)
		{
			System.out.println(rse);
			rse.printStackTrace();
		}
	}
}
