/**
 *    Free Tide Mini is a free harmonic tide predictor
 *    Copyright (C) 2008  Regis COSNIER
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package freetide.mini;

import freetide.data.HarmonicIndexFile;
import freetide.util.FileBrowserEx;
import java.io.IOException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

/**
 * @author Dgis
 */
public class FreeTideMIDlet extends MIDlet implements CommandListener
{
    private boolean midletPaused = false;
	private TideLib tideLib = new TideLib();
	private TideLib pendingTideLib = null;
	private Configuration configuration = new Configuration();

	//<editor-fold defaultstate="collapsed" desc=" Generated Fields ">//GEN-BEGIN:|fields|0|
	private Command exitCommand;
	private Command aboutCommand;
	private Command okCommand;
	private Command configCommand;
	private Command cancelCommand;
	private Command selectHarmonicFileCommand;
	private FileBrowserEx fileBrowserEx;
	private Form filesLocationForm;
	private ChoiceGroup choiceGroupConfigHarmonicFileType;
	private TideCanvas tideCanvas;
	private Form aboutForm;
	private StringItem stringItem;
	private Form configForm;
	private ChoiceGroup choiceGroupStations;
	private ChoiceGroup choiceGroupClockType;
	private ChoiceGroup choiceGroupUnits;
	private StringItem stringItemConfigHarmonicIndexFile;
	private Spacer spacer;
	private ChoiceGroup choiceGroupConfigLanguage;
	private Font font1;
	//</editor-fold>//GEN-END:|fields|0|

    /**
     * The FreeTideMIDlet constructor.
     */
    public FreeTideMIDlet() {
		configuration.load();
		switch(configuration.language)
		{
			case Configuration.LANGUAGE_ENGLISH: LocalizationSupport.initLocalizationSupport(null); break;
			case Configuration.LANGUAGE_FRENCH: LocalizationSupport.initLocalizationSupport("fr"); break;
		}
    }

	//<editor-fold defaultstate="collapsed" desc=" Generated Methods ">//GEN-BEGIN:|methods|0|
	//</editor-fold>//GEN-END:|methods|0|

	//<editor-fold defaultstate="collapsed" desc=" Generated Method: initialize ">//GEN-BEGIN:|0-initialize|0|0-preInitialize
	/**
	 * Initilizes the application.
	 * It is called only once when the MIDlet is started. The method is called before the <code>startMIDlet</code> method.
	 */
	private void initialize() {//GEN-END:|0-initialize|0|0-preInitialize
        // write pre-initialize user code here
//GEN-LINE:|0-initialize|1|0-postInitialize
        // write post-initialize user code here
	}//GEN-BEGIN:|0-initialize|2|
	//</editor-fold>//GEN-END:|0-initialize|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Method: startMIDlet ">//GEN-BEGIN:|3-startMIDlet|0|3-preAction
	/**
	 * Performs an action assigned to the Mobile Device - MIDlet Started point.
	 */
	public void startMIDlet() {//GEN-END:|3-startMIDlet|0|3-preAction
        // write pre-action user code here
		switchDisplayable(null, getTideCanvas());//GEN-LINE:|3-startMIDlet|1|3-postAction
        // write post-action user code here
		tideCanvas.setLanguage(configuration.language);
		tideCanvas.setUnit(configuration.unit);
		tideCanvas.setClockType(configuration.clockType);
		loadTide(tideLib, configuration.harmonicFilePath);
		showLocation();
	}//GEN-BEGIN:|3-startMIDlet|2|
	//</editor-fold>//GEN-END:|3-startMIDlet|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Method: resumeMIDlet ">//GEN-BEGIN:|4-resumeMIDlet|0|4-preAction
	/**
	 * Performs an action assigned to the Mobile Device - MIDlet Resumed point.
	 */
	public void resumeMIDlet() {//GEN-END:|4-resumeMIDlet|0|4-preAction
        // write pre-action user code here
//GEN-LINE:|4-resumeMIDlet|1|4-postAction
        // write post-action user code here
	}//GEN-BEGIN:|4-resumeMIDlet|2|
	//</editor-fold>//GEN-END:|4-resumeMIDlet|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Method: switchDisplayable ">//GEN-BEGIN:|5-switchDisplayable|0|5-preSwitch
	/**
	 * Switches a current displayable in a display. The <code>display</code> instance is taken from <code>getDisplay</code> method. This method is used by all actions in the design for switching displayable.
	 * @param alert the Alert which is temporarily set to the display; if <code>null</code>, then <code>nextDisplayable</code> is set immediately
	 * @param nextDisplayable the Displayable to be set
	 */
	public void switchDisplayable(Alert alert, Displayable nextDisplayable) {//GEN-END:|5-switchDisplayable|0|5-preSwitch
        // write pre-switch user code here
		Display display = getDisplay();//GEN-BEGIN:|5-switchDisplayable|1|5-postSwitch
		if (alert == null) {
			display.setCurrent(nextDisplayable);
		} else {
			display.setCurrent(alert, nextDisplayable);
		}//GEN-END:|5-switchDisplayable|1|5-postSwitch
        // write post-switch user code here
	}//GEN-BEGIN:|5-switchDisplayable|2|
	//</editor-fold>//GEN-END:|5-switchDisplayable|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Method: commandAction for Displayables ">//GEN-BEGIN:|7-commandAction|0|7-preCommandAction
	/**
	 * Called by a system to indicated that a command has been invoked on a particular displayable.
	 * @param command the Command that was invoked
	 * @param displayable the Displayable where the command was invoked
	 */
	public void commandAction(Command command, Displayable displayable) {//GEN-END:|7-commandAction|0|7-preCommandAction
        // write pre-action user code here
		if (displayable == aboutForm) {//GEN-BEGIN:|7-commandAction|1|183-preAction
			if (command == okCommand) {//GEN-END:|7-commandAction|1|183-preAction
				// write pre-action user code here
				switchDisplayable(null, getTideCanvas());//GEN-LINE:|7-commandAction|2|183-postAction
				// write post-action user code here
			}//GEN-BEGIN:|7-commandAction|3|84-preAction
		} else if (displayable == configForm) {
			if (command == cancelCommand) {//GEN-END:|7-commandAction|3|84-preAction
				// write pre-action user code here
				switchDisplayable(null, getTideCanvas());//GEN-LINE:|7-commandAction|4|84-postAction
				// write post-action user code here
			} else if (command == okCommand) {//GEN-LINE:|7-commandAction|5|86-preAction
				validateConfigForm();
				switchDisplayable(null, getTideCanvas());//GEN-LINE:|7-commandAction|6|86-postAction
				// write post-action user code here
			} else if (command == selectHarmonicFileCommand) {//GEN-LINE:|7-commandAction|7|105-preAction
				// write pre-action user code here
				switchDisplayable(null, getFilesLocationForm());//GEN-LINE:|7-commandAction|8|105-postAction
				// write post-action user code here
			}//GEN-BEGIN:|7-commandAction|9|135-preAction
		} else if (displayable == fileBrowserEx) {
			if (command == List.SELECT_COMMAND) {//GEN-END:|7-commandAction|9|135-preAction
				// write pre-action user code here
				fileBrowserExAction();//GEN-LINE:|7-commandAction|10|135-postAction
				// write post-action user code here
			} else if (command == cancelCommand) {//GEN-LINE:|7-commandAction|11|174-preAction
				// write pre-action user code here
				switchDisplayable(null, getConfigForm());//GEN-LINE:|7-commandAction|12|174-postAction
				// write post-action user code here
			}//GEN-BEGIN:|7-commandAction|13|160-preAction
		} else if (displayable == filesLocationForm) {
			if (command == cancelCommand) {//GEN-END:|7-commandAction|13|160-preAction
				// write pre-action user code here
				switchDisplayable(null, getConfigForm());//GEN-LINE:|7-commandAction|14|160-postAction
				// write post-action user code here
			} else if (command == okCommand) {//GEN-LINE:|7-commandAction|15|161-preAction
				// write pre-action user code here
				harmonicFileLocationChoice();//GEN-LINE:|7-commandAction|16|161-postAction
				// write post-action user code here
			}//GEN-BEGIN:|7-commandAction|17|180-preAction
		} else if (displayable == tideCanvas) {
			if (command == aboutCommand) {//GEN-END:|7-commandAction|17|180-preAction
				// write pre-action user code here
				switchDisplayable(null, getAboutForm());//GEN-LINE:|7-commandAction|18|180-postAction
				// write post-action user code here
			} else if (command == configCommand) {//GEN-LINE:|7-commandAction|19|88-preAction
				// write pre-action user code here
				switchDisplayable(null, getConfigForm());//GEN-LINE:|7-commandAction|20|88-postAction
				initConfigForm();
			} else if (command == exitCommand) {//GEN-LINE:|7-commandAction|21|55-preAction
				// write pre-action user code here
				exitMIDlet();//GEN-LINE:|7-commandAction|22|55-postAction
				// write post-action user code here
			}//GEN-BEGIN:|7-commandAction|23|7-postCommandAction
		}//GEN-END:|7-commandAction|23|7-postCommandAction
        // write post-action user code here
		if (displayable == fileBrowserEx) {
			if (command == FileBrowserEx.SELECT_FILE_COMMAND)
			{
				validateHarmonicFileSelection();
			}
		}
	}//GEN-BEGIN:|7-commandAction|24|
	//</editor-fold>//GEN-END:|7-commandAction|24|



	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: exitCommand ">//GEN-BEGIN:|54-getter|0|54-preInit
	/**
	 * Returns an initiliazed instance of exitCommand component.
	 * @return the initialized component instance
	 */
	public Command getExitCommand() {
		if (exitCommand == null) {//GEN-END:|54-getter|0|54-preInit
			// write pre-init user code here
			exitCommand = new Command(LocalizationSupport.getMessage("CMD_EXIT"), Command.EXIT, 0);//GEN-LINE:|54-getter|1|54-postInit
			// write post-init user code here
		}//GEN-BEGIN:|54-getter|2|
		return exitCommand;
	}
	//</editor-fold>//GEN-END:|54-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: tideCanvas ">//GEN-BEGIN:|52-getter|0|52-preInit
	/**
	 * Returns an initiliazed instance of tideCanvas component.
	 * @return the initialized component instance
	 */
	public TideCanvas getTideCanvas() {
		if (tideCanvas == null) {//GEN-END:|52-getter|0|52-preInit
			// write pre-init user code here
			tideCanvas = new TideCanvas();//GEN-BEGIN:|52-getter|1|52-postInit
			tideCanvas.setTitle("Free Tide Mini");
			tideCanvas.addCommand(getExitCommand());
			tideCanvas.addCommand(getConfigCommand());
			tideCanvas.addCommand(getAboutCommand());
			tideCanvas.setCommandListener(this);//GEN-END:|52-getter|1|52-postInit
			// write post-init user code here
		}//GEN-BEGIN:|52-getter|2|
		return tideCanvas;
	}
	//</editor-fold>//GEN-END:|52-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: cancelCommand ">//GEN-BEGIN:|83-getter|0|83-preInit
	/**
	 * Returns an initiliazed instance of cancelCommand component.
	 * @return the initialized component instance
	 */
	public Command getCancelCommand() {
		if (cancelCommand == null) {//GEN-END:|83-getter|0|83-preInit
			// write pre-init user code here
			cancelCommand = new Command(LocalizationSupport.getMessage("CMD_CANCEL"), Command.CANCEL, 0);//GEN-LINE:|83-getter|1|83-postInit
			// write post-init user code here
		}//GEN-BEGIN:|83-getter|2|
		return cancelCommand;
	}
	//</editor-fold>//GEN-END:|83-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: okCommand ">//GEN-BEGIN:|85-getter|0|85-preInit
	/**
	 * Returns an initiliazed instance of okCommand component.
	 * @return the initialized component instance
	 */
	public Command getOkCommand() {
		if (okCommand == null) {//GEN-END:|85-getter|0|85-preInit
			// write pre-init user code here
			okCommand = new Command(LocalizationSupport.getMessage("CMD_OK"), Command.OK, 0);//GEN-LINE:|85-getter|1|85-postInit
			// write post-init user code here
		}//GEN-BEGIN:|85-getter|2|
		return okCommand;
	}
	//</editor-fold>//GEN-END:|85-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: configCommand ">//GEN-BEGIN:|87-getter|0|87-preInit
	/**
	 * Returns an initiliazed instance of configCommand component.
	 * @return the initialized component instance
	 */
	public Command getConfigCommand() {
		if (configCommand == null) {//GEN-END:|87-getter|0|87-preInit
			// write pre-init user code here
			configCommand = new Command(LocalizationSupport.getMessage("CMD_CONFIG"), Command.SCREEN, 0);//GEN-LINE:|87-getter|1|87-postInit
			// write post-init user code here
		}//GEN-BEGIN:|87-getter|2|
		return configCommand;
	}
	//</editor-fold>//GEN-END:|87-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: configForm ">//GEN-BEGIN:|80-getter|0|80-preInit
	/**
	 * Returns an initiliazed instance of configForm component.
	 * @return the initialized component instance
	 */
	public Form getConfigForm() {
		if (configForm == null) {//GEN-END:|80-getter|0|80-preInit
			// write pre-init user code here
			configForm = new Form(LocalizationSupport.getMessage("FORM_SETTINGS"), new Item[] { getSpacer(), getChoiceGroupConfigLanguage(), getChoiceGroupUnits(), getChoiceGroupClockType(), getStringItemConfigHarmonicIndexFile(), getChoiceGroupStations() });//GEN-BEGIN:|80-getter|1|80-postInit
			configForm.addCommand(getCancelCommand());
			configForm.addCommand(getOkCommand());
			configForm.addCommand(getSelectHarmonicFileCommand());
			configForm.setCommandListener(this);//GEN-END:|80-getter|1|80-postInit
			// write post-init user code here
//			long currentTime = System.currentTimeMillis();
//			Calendar currentCalendar = Calendar.getInstance();
//			long currentDate = currentCalendar.getTime().getTime();
//			getStringItem().setText("timeZone: " + currentCalendar.getTimeZone().getID() + ", " + TideCanvas.formatCalendarLong(currentCalendar));
		}//GEN-BEGIN:|80-getter|2|
		return configForm;
	}
	//</editor-fold>//GEN-END:|80-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: spacer ">//GEN-BEGIN:|96-getter|0|96-preInit
	/**
	 * Returns an initiliazed instance of spacer component.
	 * @return the initialized component instance
	 */
	public Spacer getSpacer() {
		if (spacer == null) {//GEN-END:|96-getter|0|96-preInit
			// write pre-init user code here
			spacer = new Spacer(16, 1);//GEN-LINE:|96-getter|1|96-postInit
			// write post-init user code here
		}//GEN-BEGIN:|96-getter|2|
		return spacer;
	}
	//</editor-fold>//GEN-END:|96-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: selectHarmonicFileCommand ">//GEN-BEGIN:|104-getter|0|104-preInit
	/**
	 * Returns an initiliazed instance of selectHarmonicFileCommand component.
	 * @return the initialized component instance
	 */
	public Command getSelectHarmonicFileCommand() {
		if (selectHarmonicFileCommand == null) {//GEN-END:|104-getter|0|104-preInit
			// write pre-init user code here
			selectHarmonicFileCommand = new Command(LocalizationSupport.getMessage("CMD_SELECT_HARMONICS"), Command.SCREEN, 0);//GEN-LINE:|104-getter|1|104-postInit
			// write post-init user code here
		}//GEN-BEGIN:|104-getter|2|
		return selectHarmonicFileCommand;
	}
	//</editor-fold>//GEN-END:|104-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: choiceGroupConfigLanguage ">//GEN-BEGIN:|97-getter|0|97-preInit
	/**
	 * Returns an initiliazed instance of choiceGroupConfigLanguage component.
	 * @return the initialized component instance
	 */
	public ChoiceGroup getChoiceGroupConfigLanguage() {
		if (choiceGroupConfigLanguage == null) {//GEN-END:|97-getter|0|97-preInit
			// write pre-init user code here
			choiceGroupConfigLanguage = new ChoiceGroup(LocalizationSupport.getMessage("CHOICE_LANGUAGE"), Choice.POPUP);//GEN-BEGIN:|97-getter|1|97-postInit
			choiceGroupConfigLanguage.append(LocalizationSupport.getMessage("CHOICE_LANGUAGE_ENGLISH"), null);
			choiceGroupConfigLanguage.append(LocalizationSupport.getMessage("CHOICE_LANGUAGE_FRENCH"), null);
			choiceGroupConfigLanguage.setSelectedFlags(new boolean[] { false, false });
			choiceGroupConfigLanguage.setFont(0, null);
			choiceGroupConfigLanguage.setFont(1, null);//GEN-END:|97-getter|1|97-postInit
			// write post-init user code here
		}//GEN-BEGIN:|97-getter|2|
		return choiceGroupConfigLanguage;
	}
	//</editor-fold>//GEN-END:|97-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: stringItemConfigHarmonicIndexFile ">//GEN-BEGIN:|103-getter|0|103-preInit
	/**
	 * Returns an initiliazed instance of stringItemConfigHarmonicIndexFile component.
	 * @return the initialized component instance
	 */
	public StringItem getStringItemConfigHarmonicIndexFile() {
		if (stringItemConfigHarmonicIndexFile == null) {//GEN-END:|103-getter|0|103-preInit
			// write pre-init user code here
			stringItemConfigHarmonicIndexFile = new StringItem(LocalizationSupport.getMessage("STRING_ITEM_FILE_LOCATION_TITLE"), LocalizationSupport.getMessage("STRING_ITEM_FILE_LOCATION_TEXT"), Item.PLAIN);//GEN-LINE:|103-getter|1|103-postInit
			// write post-init user code here
		}//GEN-BEGIN:|103-getter|2|
		return stringItemConfigHarmonicIndexFile;
	}
	//</editor-fold>//GEN-END:|103-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: fileBrowserEx ">//GEN-BEGIN:|134-getter|0|134-preInit
	/**
	 * Returns an initiliazed instance of fileBrowserEx component.
	 * @return the initialized component instance
	 */
	public FileBrowserEx getFileBrowserEx() {
		if (fileBrowserEx == null) {//GEN-END:|134-getter|0|134-preInit
			// write pre-init user code here
			fileBrowserEx = new FileBrowserEx(null);//GEN-BEGIN:|134-getter|1|134-postInit
			fileBrowserEx.setTitle(LocalizationSupport.getMessage("FORM_FILE_BROWSER"));
			fileBrowserEx.addCommand(getCancelCommand());
			fileBrowserEx.setCommandListener(this);//GEN-END:|134-getter|1|134-postInit
			// write post-init user code here
		}//GEN-BEGIN:|134-getter|2|
		return fileBrowserEx;
	}
	//</editor-fold>//GEN-END:|134-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Method: fileBrowserExAction ">//GEN-BEGIN:|134-action|0|134-preAction
	/**
	 * Performs an action assigned to the selected list element in the fileBrowserEx component.
	 */
	public void fileBrowserExAction() {//GEN-END:|134-action|0|134-preAction
		// enter pre-action user code here
		String __selectedString = getFileBrowserEx().getString(getFileBrowserEx().getSelectedIndex());//GEN-LINE:|134-action|1|134-postAction
		// enter post-action user code here
	}//GEN-BEGIN:|134-action|2|
	//</editor-fold>//GEN-END:|134-action|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: choiceGroupStations ">//GEN-BEGIN:|147-getter|0|147-preInit
	/**
	 * Returns an initiliazed instance of choiceGroupStations component.
	 * @return the initialized component instance
	 */
	public ChoiceGroup getChoiceGroupStations() {
		if (choiceGroupStations == null) {//GEN-END:|147-getter|0|147-preInit
			// write pre-init user code here
			choiceGroupStations = new ChoiceGroup(LocalizationSupport.getMessage("CHOICE_STATIONS"), Choice.POPUP);//GEN-LINE:|147-getter|1|147-postInit
			// write post-init user code here
		}//GEN-BEGIN:|147-getter|2|
		return choiceGroupStations;
	}
	//</editor-fold>//GEN-END:|147-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Method: harmonicFileLocationChoice ">//GEN-BEGIN:|162-switch|0|162-preSwitch
	/**
	 * Performs an action assigned to the harmonicFileLocationChoice switch-point.
	 */
	public void harmonicFileLocationChoice() {//GEN-END:|162-switch|0|162-preSwitch
		// enter pre-switch user code here
		switch (choiceGroupConfigHarmonicFileType.getSelectedIndex()) {//GEN-BEGIN:|162-switch|1|163-preAction
			case 0://GEN-END:|162-switch|1|163-preAction
				// write pre-action user code here
				switchDisplayable(null, getConfigForm());//GEN-LINE:|162-switch|2|163-postAction
				// write post-action user code here
				validateHarmonicResource();
				break;//GEN-BEGIN:|162-switch|3|165-preAction
			case 1://GEN-END:|162-switch|3|165-preAction
				// write pre-action user code here
				switchDisplayable(null, getFileBrowserEx());//GEN-LINE:|162-switch|4|165-postAction
				// write post-action user code here
				break;//GEN-BEGIN:|162-switch|5|162-postSwitch
		}//GEN-END:|162-switch|5|162-postSwitch
		// enter post-switch user code here
	}//GEN-BEGIN:|162-switch|6|
	//</editor-fold>//GEN-END:|162-switch|6|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: filesLocationForm ">//GEN-BEGIN:|156-getter|0|156-preInit
	/**
	 * Returns an initiliazed instance of filesLocationForm component.
	 * @return the initialized component instance
	 */
	public Form getFilesLocationForm() {
		if (filesLocationForm == null) {//GEN-END:|156-getter|0|156-preInit
			// write pre-init user code here
			filesLocationForm = new Form(LocalizationSupport.getMessage("FORM_TYPE_FILE"), new Item[] { getChoiceGroupConfigHarmonicFileType() });//GEN-BEGIN:|156-getter|1|156-postInit
			filesLocationForm.addCommand(getCancelCommand());
			filesLocationForm.addCommand(getOkCommand());
			filesLocationForm.setCommandListener(this);//GEN-END:|156-getter|1|156-postInit
			// write post-init user code here
		}//GEN-BEGIN:|156-getter|2|
		return filesLocationForm;
	}
	//</editor-fold>//GEN-END:|156-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: choiceGroupConfigHarmonicFileType ">//GEN-BEGIN:|157-getter|0|157-preInit
	/**
	 * Returns an initiliazed instance of choiceGroupConfigHarmonicFileType component.
	 * @return the initialized component instance
	 */
	public ChoiceGroup getChoiceGroupConfigHarmonicFileType() {
		if (choiceGroupConfigHarmonicFileType == null) {//GEN-END:|157-getter|0|157-preInit
			// write pre-init user code here
			choiceGroupConfigHarmonicFileType = new ChoiceGroup(LocalizationSupport.getMessage("CHOICE_FILE_TYPE"), Choice.EXCLUSIVE);//GEN-BEGIN:|157-getter|1|157-postInit
			choiceGroupConfigHarmonicFileType.append(LocalizationSupport.getMessage("CHOICE_FILE_TYPE_JARS"), null);
			choiceGroupConfigHarmonicFileType.append(LocalizationSupport.getMessage("CHOICE_FILE_TYPE_FILE"), null);
			choiceGroupConfigHarmonicFileType.setSelectedFlags(new boolean[] { true, false });
			choiceGroupConfigHarmonicFileType.setFont(0, null);
			choiceGroupConfigHarmonicFileType.setFont(1, null);//GEN-END:|157-getter|1|157-postInit
			// write post-init user code here
		}//GEN-BEGIN:|157-getter|2|
		return choiceGroupConfigHarmonicFileType;
	}
	//</editor-fold>//GEN-END:|157-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: choiceGroupUnits ">//GEN-BEGIN:|176-getter|0|176-preInit
	/**
	 * Returns an initiliazed instance of choiceGroupUnits component.
	 * @return the initialized component instance
	 */
	public ChoiceGroup getChoiceGroupUnits() {
		if (choiceGroupUnits == null) {//GEN-END:|176-getter|0|176-preInit
			// write pre-init user code here
			choiceGroupUnits = new ChoiceGroup(LocalizationSupport.getMessage("CHOICE_UNITS"), Choice.EXCLUSIVE);//GEN-BEGIN:|176-getter|1|176-postInit
			choiceGroupUnits.append(LocalizationSupport.getMessage("CHOICE_UNITS_METERS"), null);
			choiceGroupUnits.append(LocalizationSupport.getMessage("CHOICE_UNITS_FEET"), null);
			choiceGroupUnits.setSelectedFlags(new boolean[] { true, false });
			choiceGroupUnits.setFont(0, null);
			choiceGroupUnits.setFont(1, null);//GEN-END:|176-getter|1|176-postInit
			// write post-init user code here
		}//GEN-BEGIN:|176-getter|2|
		return choiceGroupUnits;
	}
	//</editor-fold>//GEN-END:|176-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: aboutCommand ">//GEN-BEGIN:|179-getter|0|179-preInit
	/**
	 * Returns an initiliazed instance of aboutCommand component.
	 * @return the initialized component instance
	 */
	public Command getAboutCommand() {
		if (aboutCommand == null) {//GEN-END:|179-getter|0|179-preInit
			// write pre-init user code here
			aboutCommand = new Command(LocalizationSupport.getMessage("CMD_ABOUT"), Command.SCREEN, 0);//GEN-LINE:|179-getter|1|179-postInit
			// write post-init user code here
		}//GEN-BEGIN:|179-getter|2|
		return aboutCommand;
	}
	//</editor-fold>//GEN-END:|179-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: aboutForm ">//GEN-BEGIN:|181-getter|0|181-preInit
	/**
	 * Returns an initiliazed instance of aboutForm component.
	 * @return the initialized component instance
	 */
	public Form getAboutForm() {
		if (aboutForm == null) {//GEN-END:|181-getter|0|181-preInit
			// write pre-init user code here
			aboutForm = new Form(LocalizationSupport.getMessage("FORM_ABOUT"), new Item[] { getStringItem() });//GEN-BEGIN:|181-getter|1|181-postInit
			aboutForm.addCommand(getOkCommand());
			aboutForm.setCommandListener(this);//GEN-END:|181-getter|1|181-postInit
			// write post-init user code here
		}//GEN-BEGIN:|181-getter|2|
		return aboutForm;
	}
	//</editor-fold>//GEN-END:|181-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: stringItem ">//GEN-BEGIN:|182-getter|0|182-preInit
	/**
	 * Returns an initiliazed instance of stringItem component.
	 * @return the initialized component instance
	 */
	public StringItem getStringItem() {
		if (stringItem == null) {//GEN-END:|182-getter|0|182-preInit
			// write pre-init user code here
			stringItem = new StringItem("", "Free Tide Mini is a free harmonic tide predictor for the java mobile phones (J2ME MIDP 2.0 only).\nThe official page is located here: http://regis.cosnier.free.fr/\n\nIt is a port of XTide (http://www.flaterco.com/xtide/), so it uses the regular harmonic files. Only the ASCII file is supported, the new TCD binary file is not supported yet.\nThese harmonics files could be put in the jar midlet before the installation process, or you can try to load it from the filesystem (with some permission warnings).\nBecause the RAM is very limited in J2ME platform, you should reduce the size of the files by removing the stations you do not need. Also, you can strip the comment lines from these files (The lines beginning with #).\nFor instance, I only care about the french stations, so after striping the harmonic files I get about 300Ko of data which seems to be well accepted by my mobile phone.\n\nKEYS:\nLEFT/RIGHT keys to go backward or forward in the time.\nUP or BOTTOM keys to come back now.\n\nLICENCE:\nVersion 0.1 Copyright (c) 2008 Regis Cosnier, All Rights Reserved.\nThis program is free software and may be distributed according to the terms of the GNU General Public License (GPL).\n\nCHANGES:\n\nversion 0.1 (2008-02-05)\n- First version", Item.PLAIN);//GEN-BEGIN:|182-getter|1|182-postInit
			stringItem.setLayout(ImageItem.LAYOUT_DEFAULT);
			stringItem.setFont(getFont1());//GEN-END:|182-getter|1|182-postInit
			// write post-init user code here
		}//GEN-BEGIN:|182-getter|2|
		return stringItem;
	}
	//</editor-fold>//GEN-END:|182-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: font1 ">//GEN-BEGIN:|186-getter|0|186-preInit
	/**
	 * Returns an initiliazed instance of font1 component.
	 * @return the initialized component instance
	 */
	public Font getFont1() {
		if (font1 == null) {//GEN-END:|186-getter|0|186-preInit
			// write pre-init user code here
			font1 = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_SMALL);//GEN-LINE:|186-getter|1|186-postInit
			// write post-init user code here
		}//GEN-BEGIN:|186-getter|2|
		return font1;
	}
	//</editor-fold>//GEN-END:|186-getter|2|

	//<editor-fold defaultstate="collapsed" desc=" Generated Getter: choiceGroupClockType ">//GEN-BEGIN:|187-getter|0|187-preInit
	/**
	 * Returns an initiliazed instance of choiceGroupClockType component.
	 * @return the initialized component instance
	 */
	public ChoiceGroup getChoiceGroupClockType() {
		if (choiceGroupClockType == null) {//GEN-END:|187-getter|0|187-preInit
			// write pre-init user code here
			choiceGroupClockType = new ChoiceGroup(LocalizationSupport.getMessage("CHOICE_CLOCK"), Choice.EXCLUSIVE);//GEN-BEGIN:|187-getter|1|187-postInit
			choiceGroupClockType.append(LocalizationSupport.getMessage("CHOICE_CLOCK_12"), null);
			choiceGroupClockType.append(LocalizationSupport.getMessage("CHOICE_CLOCK_24"), null);
			choiceGroupClockType.setSelectedFlags(new boolean[] { false, true });
			choiceGroupClockType.setFont(0, null);
			choiceGroupClockType.setFont(1, null);//GEN-END:|187-getter|1|187-postInit
			// write post-init user code here
		}//GEN-BEGIN:|187-getter|2|
		return choiceGroupClockType;
	}
	//</editor-fold>//GEN-END:|187-getter|2|




    /**
     * Returns a display instance.
     * @return the display instance.
     */
    public Display getDisplay () {
        return Display.getDisplay(this);
    }

    /**
     * Exits MIDlet.
     */
    public void exitMIDlet() {
        switchDisplayable (null, null);
        destroyApp(true);
        notifyDestroyed();
    }

    /**
     * Called when MIDlet is started.
     * Checks whether the MIDlet have been already started and initialize/starts or resumes the MIDlet.
     */
    public void startApp() {
        if (midletPaused) {
            resumeMIDlet ();
        } else {
            initialize ();
            startMIDlet ();
        }
        midletPaused = false;
    }

    /**
     * Called when MIDlet is paused.
     */
    public void pauseApp() {
        midletPaused = true;
    }

    /**
     * Called to signal the MIDlet to terminate.
     * @param unconditional if true, then the MIDlet has to be unconditionally terminated and all resources has to be released.
     */
    public void destroyApp(boolean unconditional) {
    }

	private void loadTide(TideLib tideLibrary, String harmonicFileURL)
	{
		try
		{
			int pos = harmonicFileURL.lastIndexOf('/');
			if(pos != -1)
			{
				String path = harmonicFileURL.substring(0, pos + 1);
				String harmonicFile = harmonicFileURL.substring(pos + 1);

				tideLibrary.clear();
				tideLibrary.loadHarmonicIndex(path, harmonicFile);
			}
		}
		catch(OutOfMemoryError ex)
		{
			ShowAlert(LocalizationSupport.getMessage("TEXT_EXCEPTION"), LocalizationSupport.getMessage("TEXT_EXCEPTION1"));
			ex.printStackTrace();
		}
		catch(Exception ex)
		{
			ShowAlert(LocalizationSupport.getMessage("TEXT_EXCEPTION"), LocalizationSupport.getMessage("TEXT_EXCEPTION2") + ex.getMessage());
			ex.printStackTrace();
		}
	}
	
	private void populateListStation(TideLib tideLibrary)
	{
		getChoiceGroupStations();
		choiceGroupStations.deleteAll();
		HarmonicIndexFile harmonicIndexFile = tideLibrary.getHarmonicIndexFile();
		int locationNum = harmonicIndexFile.getLocationNum();
		for(int i = 0; i < locationNum; i++)
		{
			choiceGroupStations.append(harmonicIndexFile.getLocation(i).getLocationName(), null);
		}
	}
		
	private void ShowAlert(String title, String text)
	{
		Alert alert = new Alert(title, text, null, AlertType.INFO);
		alert.setTimeout(Alert.FOREVER);
		Display.getDisplay(this).setCurrent(alert);
	}
	
	private void initConfigForm()
	{
		// write post-action user code here
		choiceGroupConfigLanguage.setSelectedIndex(configuration.language, true);
		choiceGroupUnits.setSelectedIndex(configuration.unit, true);
		stringItemConfigHarmonicIndexFile.setText(configuration.harmonicFilePath);
		populateListStation(tideLib);
		choiceGroupStations.setSelectedIndex(configuration.lastStation, true);
	}
	
	private void validateHarmonicFileSelection()
	{
		stringItemConfigHarmonicIndexFile.setText(fileBrowserEx.getSelectedFileURL());
		changeHarmonicIndexFile();
	}
	
	private void validateHarmonicResource()
	{
		stringItemConfigHarmonicIndexFile.setText("/Harmonic.idx");
		changeHarmonicIndexFile();
	}
	
	private void validateConfigForm()
	{
		// write pre-action user code here
		configuration.language = (byte) choiceGroupConfigLanguage.getSelectedIndex();
		configuration.harmonicFilePath = stringItemConfigHarmonicIndexFile.getText();
		configuration.lastStation = choiceGroupStations.getSelectedIndex();
		configuration.unit = choiceGroupUnits.getSelectedIndex();
		configuration.clockType = choiceGroupClockType.getSelectedIndex();

		if(pendingTideLib != null)
		{
			tideLib = pendingTideLib;
			pendingTideLib = null;
		}

		switch(configuration.language)
		{
			case Configuration.LANGUAGE_ENGLISH: LocalizationSupport.initLocalizationSupport(null); break;
			case Configuration.LANGUAGE_FRENCH: LocalizationSupport.initLocalizationSupport("fr"); break;
		}
		tideCanvas.setLanguage(configuration.language);
		tideCanvas.setUnit(configuration.unit);
		tideCanvas.setClockType(configuration.clockType);
		
		showLocation();
		configuration.save();
	}
	
	private	void changeHarmonicIndexFile()
	{
		new Thread(new Runnable()
		{
			public void run()
			{
				pendingTideLib = new TideLib();
				loadTide(pendingTideLib, choiceGroupConfigHarmonicFileType.getSelectedIndex() == 0 ? "/Harmonic.idx" : stringItemConfigHarmonicIndexFile.getText());
				populateListStation(pendingTideLib);
				switchDisplayable(null, getConfigForm());
			}
		}).start();
	}
	
	private	void showLocation()
	{
		getTideCanvas().setWaitingMessage(LocalizationSupport.getMessage("TEXT_WAITING2"));
		getTideCanvas().setLocation(null);
		new Thread(new Runnable()
		{

			public
			void run()
			{
				try
				{
					tideLib.setCurrentLocation(tideLib.getHarmonicIndexFile().getLocation(configuration.lastStation));
					getTideCanvas().setLocation(tideLib.getCurrentLocation());
				}
				catch (IOException ex)
				{
					ShowAlert(LocalizationSupport.getMessage("TEXT_EXCEPTION"), ex.getMessage());
					ex.printStackTrace();
				}
			}
		}).start();
	}
}
