/**
 *    Free Tide Mini is a free harmonic tide predictor
 *    Copyright (C) 2008  Regis COSNIER
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package freetide.mini;

import freetide.data.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import javax.microedition.io.Connector;
import javax.microedition.io.InputConnection;

/**
 *
 * @author Dgis
 */
public class TideLib
{
	private HarmonicIndexFile harmonicIndexFile;
	private Location currentLocation;
	private String harmonicIndexFilePath;
	private boolean isHarmonicInResource = false;
	
	public Location getCurrentLocation() { return currentLocation; }
	public void setCurrentLocation(Location location) throws IOException
	{
		this.currentLocation = location;
		if(!location.isLoaded())
			loadLocation(location);
	}
	public HarmonicIndexFile getHarmonicIndexFile() { return harmonicIndexFile; }

	public TideLib()
	{
	}
	
	public void clear()
	{
		harmonicIndexFile = null;
		currentLocation = null;
		harmonicIndexFilePath = null;
		isHarmonicInResource = false;
	}
	public void loadHarmonicIndex(String harmonicIndexFilePath, String harmonicIndexFileName) throws IOException
	{
		this.harmonicIndexFilePath = harmonicIndexFilePath;
		isHarmonicInResource = !harmonicIndexFilePath.startsWith("file://");
		InputStream inputStream;
		if(isHarmonicInResource)
			inputStream = getClass().getResourceAsStream(harmonicIndexFilePath + harmonicIndexFileName);
		else
		{
			InputConnection connection = (InputConnection)Connector.open(harmonicIndexFilePath + harmonicIndexFileName, Connector.READ);
			inputStream = connection.openInputStream();
		}
		InputStreamReader inputStreamReader = new InputStreamReader(inputStream); //, "ISO8859_1");
		harmonicIndexFile = new HarmonicIndexFile();
		try
		{
			harmonicIndexFile.loadIndex(inputStreamReader);
		}
		finally
		{
			inputStream.close();
		}
	}
	
	public void loadLocation(Location location) throws IOException
	{
		HarmonicFile harmonicFile = location.getHarmonicFile();
		if(!harmonicFile.isLoaded())
		{
			InputStream inputStream;
			if(isHarmonicInResource)
				inputStream = getClass().getResourceAsStream(harmonicIndexFilePath + harmonicFile.getFileName());
			else
			{
				InputConnection connection = (InputConnection)Connector.open(harmonicIndexFilePath + harmonicFile.getFileName(), Connector.READ);
				inputStream = connection.openInputStream();
			}
			InputStreamReader inputStreamReader = new InputStreamReader(inputStream); //, "ISO8859_1");
			try
			{
				harmonicFile.loadConstituents(inputStreamReader);
			}
			finally
			{
				inputStream.close();
			}
		}
		if(harmonicFile.isLoaded())
		{
			InputStream inputStream;
			if(isHarmonicInResource)
				inputStream = getClass().getResourceAsStream(harmonicIndexFilePath + harmonicFile.getFileName());
			else
			{
				InputConnection connection = (InputConnection)Connector.open(harmonicIndexFilePath + harmonicFile.getFileName(), Connector.READ);
				inputStream = connection.openInputStream();
			}
			InputStreamReader inputStreamReader = new InputStreamReader(inputStream); //, "ISO8859_1");
			try
			{
				harmonicFile.loadLocation(inputStreamReader, location);
			}
			finally
			{
				inputStream.close();
			}
		}
	}
}
